@@ -44,6 +44,7 @@ class ZPush {
     const ASV_12 = "12.0";
     const ASV_121 = "12.1";
     const ASV_14 = "14.0";
+    const ASV_141 = "14.1";
 
     /**
      * Command codes for base64 encoded requests (AS >= 12.1)
@@ -95,7 +96,8 @@ class ZPush {
     static private $supportedASVersions = array(
                     self::ASV_12,
                     self::ASV_121,
-                    self::ASV_14
+                    self::ASV_14,
+                    self::ASV_141
                 );
 
     static private $supportedCommands = array(
@@ -283,6 +285,33 @@ class ZPush {
                 throw new FatalMisconfigurationException(sprintf("Your policies' configuration file doesn't contain the required [default] section. Please check the '%s' file.", $policyfile));
             }
         }
+
+        if (defined('USE_X_FORWARDED_FOR_HEADER')) {
+            ZLog::Write(LOGLEVEL_INFO, "The configuration parameter 'USE_X_FORWARDED_FOR_HEADER' was deprecated in favor of 'USE_CUSTOM_REMOTE_IP_HEADER'. Please update your configuration.");
+        }
+
+        //check folder re-sync triggering settings
+        if (defined('DELETION_COUNT_THR') && !defined('DELETION_RATIO_THR')) {
+            throw new FatalMisconfigurationException("Only DELETION_COUNT_THR defined. Please define DELETION_RATIO_THR.");
+        }
+        elseif (!defined('DELETION_COUNT_THR') && defined('DELETION_RATIO_THR')) {
+            throw new FatalMisconfigurationException("Only DELETION_RATIO_THR defined. Please define DELETION_COUNT_THR.");
+        }
+        if ((defined('DELETION_COUNT_THR')) && (!is_int(DELETION_COUNT_THR) || DELETION_COUNT_THR < 1)) {
+            throw new FatalMisconfigurationException("The DELETION_COUNT_THR value must be a number higher than 0.");
+        }
+        if ((defined('DELETION_RATIO_THR')) && (!is_numeric(DELETION_RATIO_THR) || DELETION_RATIO_THR <= 0)) {
+            throw new FatalMisconfigurationException("The DELETION_RATIO_THR value must be a number higher than 0.");
+        }
+
+        //check retry loop settings when writing file state machine data to disk
+        if ((defined('FILE_STATE_WRITE_ATTEMPTS')) && (!is_int(FILE_STATE_WRITE_ATTEMPTS) || FILE_STATE_WRITE_ATTEMPTS < 1)) {
+            throw new FatalMisconfigurationException("The FILE_STATE_WRITE_ATTEMPTS value must be a number higher than 0.");
+        }
+        if ((defined('FILE_STATE_WRITE_SLEEP')) && (!is_int(FILE_STATE_WRITE_SLEEP) || FILE_STATE_WRITE_SLEEP < 1)) {
+            throw new FatalMisconfigurationException("The FILE_STATE_WRITE_SLEEP value must be a number higher than 0.");
+        }        
+
         return true;
     }
 
@@ -460,6 +489,21 @@ class ZPush {
         return ZPush::$deviceManager;
     }
 
+    /**
+     * Load another device in the DeviceManager and return it.
+     *
+     * @param ASDevice $asDevice
+     * @param boolean $initialize - default: true
+     * @return DeviceManager
+     */
+    static public function GetDeviceManagerWithDevice($asDevice, $initialize = true) {
+        $dm = ZPush::GetDeviceManager($initialize);
+        if ($dm) {
+            $dm->SetDevice($asDevice);
+        }
+        return $dm;
+    }
+
     /**
      * Returns the Top data collector object
      *
@@ -677,7 +721,13 @@ class ZPush {
                         continue;
                     }
 
-                    $folder = self::GetDeviceManager()->BuildSyncFolderObject($af['store'], $af['folderid'], '0', $af['name'], $af['type'], 0, DeviceManager::FLD_ORIGIN_CONFIG);
+                    // don't fail hard if no flags are set, but we at least warn about it
+                    if (!isset($af['flags'])) {
+                        ZLog::Write(LOGLEVEL_WARN, sprintf("ZPush::getAddSyncFolders() : the additional folder '%s' is not configured completely. Missing 'flags' parameter, defaulting to DeviceManager::FLD_FLAGS_NONE.", $af['name']));
+                        $af['flags'] = DeviceManager::FLD_FLAGS_NONE;
+                    }
+
+                    $folder = self::GetDeviceManager()->BuildSyncFolderObject($af['store'], $af['folderid'], '0', $af['name'], $af['type'], $af['flags'], DeviceManager::FLD_ORIGIN_CONFIG);
                     self::$addSyncFolders[$folder->BackendId] = $folder;
                 }
             }
