@@ -885,19 +885,24 @@ class Utils {
      * @return boolean
      */
     public static function FixFileOwner($file) {
-        if(posix_getuid() == 0 && is_file($file)) {
+        if (!function_exists('posix_getuid')) {
+           ZLog::Write(LOGLEVEL_DEBUG, "Utils::FixeFileOwner(): Posix subsystem not available, skipping.");
+           return false;
+        }
+        if (posix_getuid() == 0 && is_file($file)) {
             $dir = dirname($file);
             $perm_dir = stat($dir);
             $perm_file = stat($file);
 
-            if ($perm_file['uid'] == 0 && $perm_dir['uid'] == 0) {
+            if ($perm_file['uid'] == 0 && $perm_dir['uid'] == 0 && $perm_dir['gid'] == 0) {
                 unlink($file);
-                throw new FatalException("FixFileOwner: $dir must be owned by the nginx/apache/php user instead of root");
+                throw new FatalException("FixFileOwner: $dir must be owned by the nginx/apache/php user instead of root for debian based systems and by root:z-push for RHEL-based systems");
             }
 
             if($perm_dir['uid'] !== $perm_file['uid'] || $perm_dir['gid'] !== $perm_file['gid']) {
                 chown($file, $perm_dir['uid']);
                 chgrp($file, $perm_dir['gid']);
+                chmod($file, 0664);
             }
         }
         return true;
@@ -952,12 +957,26 @@ class Utils {
     public static function SafePutContents($filename, $data) {
         //put the 'tmp' as a prefix (and not suffix) so all glob call will not see temp files
         $tmp = dirname($filename).DIRECTORY_SEPARATOR.'tmp-'.getmypid().'-'.basename($filename);
-        if (($res = file_put_contents($tmp, $data)) !== false) {
-            self::FixFileOwner($tmp);
-            if (rename($tmp, $filename) !== true)
-                $res = false;
-        }
 
+        //number of attempts
+        $attempts = (defined('FILE_STATE_WRITE_ATTEMPTS') ? FILE_STATE_WRITE_ATTEMPTS : 3);
+        //ms to sleep between attempts
+        $sleep_time = (defined('FILE_STATE_WRITE_SLEEP') ? FILE_STATE_WRITE_SLEEP : 100);
+        $i = 1;
+        while (($i <= $attempts) && (($res = file_put_contents($tmp, $data)) === false)) {
+            ZLog::Write(LOGLEVEL_WARN, sprintf("Utils->SafePutContents: Failed on writing data in tmp - attempt: %d - filename: %s", $i, $tmp));
+            $i++;
+            usleep($sleep_time * 1000);
+        }
+        if ($res !== false){
+            self::FixFileOwner($tmp);
+            $i = 1;
+            while (($i <= $attempts) && (($res = rename($tmp, $filename)) !== true)) {
+                ZLog::Write(LOGLEVEL_WARN, sprintf("Utils->SafePutContents: Failed on rename - attempt: %d - filename: %s", $i, $tmp));
+                $i++;
+                usleep($sleep_time * 1000);
+            }      
+        }
         return $res;
     }
 
@@ -1085,6 +1104,7 @@ class Utils {
             case DeviceManager::FLD_ORIGIN_GAB:
             case DeviceManager::FLD_ORIGIN_SHARED:
             case DeviceManager::FLD_ORIGIN_USER:
+            case DeviceManager::FLD_ORIGIN_IMPERSONATED:
                 return $origin;
         }
         ZLog::Write(LOGLEVEL_WARN, sprintf("Utils->GetFolderOriginFromId(): Unknown folder origin for folder with id '%s'", $folderid));
@@ -1110,6 +1130,8 @@ class Utils {
                 return 'shared';
             case DeviceManager::FLD_ORIGIN_USER:
                 return 'user';
+            case DeviceManager::FLD_ORIGIN_IMPERSONATED:
+                return 'impersonated';
         }
         ZLog::Write(LOGLEVEL_WARN, sprintf("Utils->GetFolderOriginStringFromId(): Unknown folder origin for folder with id '%s'", $folderid));
         return 'unknown';
@@ -1354,7 +1376,7 @@ class Utils {
         if (!$rawheaders) {
             return;
         }
-        $message->headers["subject"] = Utils::convertRawHeader2Utf8($rawheaders["subject"], $message->headers["subject"]);
+        $message->headers["subject"] = isset($message->headers["subject"]) ? Utils::convertRawHeader2Utf8($rawheaders["subject"], $message->headers["subject"]) : "";
         $message->headers["from"] = Utils::convertRawHeader2Utf8($rawheaders["from"], $message->headers["from"]);
     }
 }
